

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <GL/glut.h>

#include "rubikcube.h"
#include "main.h"



RubikCube::RubikCube(int size)
{
		//Save size of cube
		this->size = size;
		this->highlight = 0;
		this->rothigh = 0;

		//Create the 3-dim. array. and initialize with black
		elmcol = new Color**[6];

		for(int i=0; i<6; i++)
		{
			elmcol[i] = new Color*[size];

			for(int j=0; j<size; j++)
				elmcol[i][j] = new Color[size];
		}

		//Reset all colors
		reset();
}


RubikCube::~RubikCube()
{
		//Delete the 3-dim. array
		for(int i=0; i<6; i++)
		{
			for(int j=0; j<size; j++)
				delete[] elmcol[i][j];

			delete[] elmcol[i];
		}
		
		delete[] elmcol;
}


void RubikCube::reset()
{
		//Set all colors
		for( int s=0; s<6; s++)
			for(int i=0; i<size; i++)
				for(int j=0; j<size; j++)
					elmcol[s][i][j] = static_cast<Color>( 1 + s );
}


void RubikCube::rotateCol()
{
		//Get the dimension - its orthogonal complement is parallel to the highlighted slice
		int highdim = highlight / size;
		//The number of slice in this dimension
		int highnr = highlight % size;


		//Rotate in x-direction (Rotate back, front, ., ., bottom, top)
		//x-coord is highnr
		if( highdim == 0 )
		{
			Color* save = new Color[size];
			memcpy(save, elmcol[4][highnr], sizeof(Color)*size);

			//Back --> Bottom
			for( int i=0; i< size; i++ )
				elmcol[4][highnr][i] = elmcol[0][highnr][size-1-i];

			//Top --> Back
			for( int i=0; i< size; i++ )
				elmcol[0][highnr][i] = elmcol[5][highnr][i];

			//Front --> Top
			for( int i=0; i< size; i++ )
				elmcol[5][highnr][i] = elmcol[1][highnr][size-1-i];

			//Bottom --> Front
			for( int i=0; i< size; i++ )
				elmcol[1][highnr][i] = save[i];

			delete [] save;
		}

		//Rotate in y-direction (Rotate back, front, left, right, ., .)
		//y-coord is highnr
		if( highdim == 1 )
		{
			Color* save = new Color[size];
			memcpy(save, elmcol[3][highnr], sizeof(Color)*size);

			//Front --> Right
			for( int i=0; i< size; i++ )
				elmcol[3][highnr][i] = elmcol[1][size-1-i][highnr];

			//Left --> Front
			for( int i=0; i< size; i++ )
				elmcol[1][i][highnr] = elmcol[2][highnr][i];

			//Back --> Left
			for( int i=0; i< size; i++ )
				elmcol[2][highnr][i] = elmcol[0][size-1-i][highnr];

			//Right --> Back
			for( int i=0; i< size; i++ )
				elmcol[0][i][highnr] = save[i];


			delete [] save;
		}
		
		//Rotate in z-direction (Rotate ., ., left, right, bottom, top)
		//z-coord is highnr
		if( highdim == 2 )
		{
			Color* save = new Color[size];
			
			for( int i=0; i< size; i++ )
				save[i] = elmcol[5][i][highnr];

			//Left --> Top
			for( int i=0; i< size; i++ )
				elmcol[5][i][highnr] = elmcol[2][i][highnr];

			//Bottom --> Left
			for( int i=0; i< size; i++ )
				elmcol[2][i][highnr] = elmcol[4][size-1-i][highnr];

			//Right --> Bottom
			for( int i=0; i< size; i++ )
				elmcol[4][i][highnr] = elmcol[3][i][highnr];

			// Top --> Right
			for( int i=0; i< size; i++ )
				elmcol[3][i][highnr] = save[size-1-i];

			delete [] save;
		}


		//Rotate a side of cube
		if( highnr == 0 || highnr == size-1 )
		{
				int side;

				//Get the side
				if( highdim==0 )
					side = highnr==0 ? 2 : 3;
				if( highdim==1 )
					side = highnr==0 ? 4 : 5;
				if( highdim==2 )
					side = highnr==0 ? 0 : 1;


				//Save old values
				Color** save = new Color*[size];			

				//Save content
				for(int i=0; i<size; i++)
				{
					save[i] = new Color[size];
					for(int j=0; j<size; j++)
						save[i][j] = elmcol[side][i][j];
				}
						
				//Rotate data
				for( int j=0; j<size; j++)
				{
					for( int i=0; i<size; i++)
						elmcol[side][i][j] = save[size-1-j][i];

					delete[] save[size-1-j];
				}

				delete[] save;
		}
}


void RubikCube::glRotate(int msdelay, int fps)
{
		const float framedelay = 1.0/fps;
		const float nrFrames = (msdelay /1000.0) / framedelay;
		const float degPerFrame = 90.0 / nrFrames;
		rothigh = 0;


		while( rothigh < 90.0 )		
		{
				usleep((int)(framedelay*1000*1000));
				rothigh += degPerFrame;
				Display();
				glutPostRedisplay();
		}


		//Rotate the color array
		rotateCol();
		rothigh = 0;
}


void RubikCube::glDisplay()
{
		//Get the dimension - its orthogonal complement is parallel to the highlighted slice
		int highdim = highlight / size;
		//The number of slice in this dimension
		int highnr = highlight % size;
		//True if highlighted sub-cube
		bool enlight;



		glDisable(GL_TEXTURE_2D);

		//Print every 
		for( int x=0; x<size; x++)
			for(int y=0; y<size; y++)
				for( int z=0; z<size; z++)
				{
						//Should sub-cube be enlighted?
						enlight = 	highdim==0 && highnr == x ||
									highdim==1 && highnr == y ||
									highdim==2 && highnr == z;

						glPushMatrix();

						//Rotate highlighted slice
						if( enlight )
						{
								if( highdim==0 )
									glRotatef(-rothigh, 1, 0, 0);
								if( highdim==1 )
									glRotatef(-rothigh, 0, -1, 0);
								if( highdim==2 )
									glRotatef(-rothigh, 0, 0, 1);
						}


						//Draw the sub-cube at postion x,y,z
						glDrawCube(x, y, z, 
										z==0 ? elmcol[0][x][y] : black,
										z==size-1 ? elmcol[1][x][y] : black,
										x==0 ? elmcol[2][y][z] : black,
										x==size-1 ? elmcol[3][y][z] : black,
										y==0 ? elmcol[4][x][z] : black,
										y==size-1 ? elmcol[5][x][z] : black,
										enlight
										);

						glPopMatrix();
				}
}


void RubikCube::glDrawCube(int x, int y, int z,
				Color back, Color front, Color left, Color right, Color bottom, Color top,
				bool highlight)
{
		//Offsets of sub-cubes
		float fx, fy, fz;

		//Element-size (size of subcube)
		const float es = 1.0/size;
		const float halfgapfac = 0.1;
		//Netto size - to get gaps
		const float esn = es*(1 - 2*halfgapfac);

		fx = x * es-0.5 + es*halfgapfac;
		fy = y * es-0.5 + es*halfgapfac;
		fz = z * es-0.5 + es*halfgapfac;

		//bottom face
		glBegin(GL_QUADS);
			glSetColor(bottom, highlight);
			glNormal3f( 0, 0, -1 );
			glVertex3f( fx + 0.0, fy + 0.0, fz + 0.0);
			glVertex3f( fx + esn, fy + 0.0, fz + 0.0);
			glVertex3f( fx + esn, fy + 0.0, fz + esn);
			glVertex3f( fx + 0.0, fy + 0.0, fz + esn);
		glEnd( );

		//top face
		glBegin(GL_QUADS);
			glSetColor(top, highlight);
			glNormal3f( 0, 0, 1 );
			glVertex3f( fx + 0.0, fy + esn, fz + 0.0);
			glVertex3f( fx + 0.0, fy + esn, fz + esn);
			glVertex3f( fx + esn, fy + esn, fz + esn);
			glVertex3f( fx + esn, fy + esn, fz + 0.0);
		glEnd( );

		//left face
		glBegin(GL_QUADS);
			glSetColor(left, highlight);
			glNormal3f( -1, 0, 0 );
			glVertex3f( fx + 0.0, fy + 0.0, fz + 0.0);
			glVertex3f( fx + 0.0, fy + 0.0, fz + esn);
			glVertex3f( fx + 0.0, fy + esn, fz + esn);
			glVertex3f( fx + 0.0, fy + esn, fz + 0.0);
	
		//right face
		glBegin(GL_QUADS);
			glSetColor(right, highlight);
			glNormal3f( 1, 0, 0 );
			glVertex3f( fx + esn, fy + 0.0, fz + 0.0);
			glVertex3f( fx + esn, fy + esn, fz + 0.0);
			glVertex3f( fx + esn, fy + esn, fz + esn);
			glVertex3f( fx + esn, fy + 0.0, fz + esn);
		glEnd( );	

		//back face
		glBegin(GL_QUADS);
			glSetColor(back, highlight);
			glNormal3f( 0, 0, -1 );
			glVertex3f( fx + 0.0, fy + 0.0, fz + 0.0);
			glVertex3f( fx + 0.0, fy + esn, fz + 0.0);
			glVertex3f( fx + esn, fy + esn, fz + 0.0);
			glVertex3f( fx + esn, fy + 0.0, fz + 0.0);
		glEnd( );

		//front face
		glBegin(GL_QUADS);
			glSetColor(front, highlight);
			glNormal3f( 0, 0, 1 );
			glVertex3f( fx + 0.0, fy + 0.0, fz + esn);
			glVertex3f( fx + esn, fy + 0.0, fz + esn);
			glVertex3f( fx + esn, fy + esn, fz + esn);
			glVertex3f( fx + 0.0, fy + esn, fz + esn);
		glEnd( );

}


void RubikCube::glSetColor(Color col, bool highlight )
{
		//Material of cubes
		static float mat[] = {0, 0, 0, 1};
		static float actual[] = {0, 0, 0, 1};


		switch( col )
		{
			case black:

				mat[0] =  0.4;
				mat[1] =  0.4;
				mat[2] =  0.4;
				break;

			case white:

				mat[0] =  1;
				mat[1] =  1;
				mat[2] =  1;
				break;

			case red:
				mat[0] =  1;
				mat[1] =  0;
				mat[2] =  0;
				break;

			case green:
				mat[0] =  0;
				mat[1] =  1;
				mat[2] =  0;
				break;

			case blue:
				mat[0] =  0;
				mat[1] =  0;
				mat[2] =  1;
				break;

			case yellow:
				mat[0] =  0;
				mat[1] =  1;
				mat[2] =  1;
				break;
			
			case pink:
				mat[0] =  1;
				mat[1] =  0;
				mat[2] =  1;
				break;
		}



		//From all directions to all directions
		actual[0] = mat[0];
		actual[1] = mat[1];
		actual[2] = mat[2];
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, actual);

		//Depends on direction
		actual[0] = 0.1*mat[0];
		actual[1] = 0.1*mat[1];
		actual[2] = 0.1*mat[2];
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, actual);


		if( !highlight )
			actual[0] = actual[1] = actual[2] = 0;
		else
		{
			actual[0] = 0.8*mat[0];
			actual[1] = 0.8*mat[1];
			actual[2] = 0.8*mat[2];
		}

		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, actual);
}





