#ifndef RUBIKCUBE_H
#define RUBIKCUBE_H


/** The common colors of sub-cubes of a rubik's cube */
enum Color
{
		black=0,
		white,
		red,
		green,
		blue,
		yellow,
		pink
};



/**
 * A rubik cube of specfic size.
 */
class RubikCube
{
private:

		/** The size of the cube */
		int size;
		/** The colors of the elements (sub cubes)
		 * elmcol[side], side=0,..,5 is the 2-dim. array of colors for specfic side.
		 * */
		Color*** elmcol;

		/** The ring which should be highlighted. Element of {0, ..., size*3}, 
		 * size-many for dimension, size-many for ring in dimension.*/
		int highlight;

		/** Angel by with the highlighted sub-cubes are rotated. */
		float rothigh;

public:

		RubikCube(int size=3);
		~RubikCube();

		/** Highlight next */
		void hightlightNext()	{ highlight++; highlight %= 3*size;  };

		/** Rotate highlighted part */
		void glRotate(int msdelay=250, int fps=20);
		/** Display cube */
		void glDisplay();


protected:
		/** Reset to start configuration */
		void reset();
		/** Rotate colors */
		void rotateCol();

		/**Draw a single cube*/
		void glDrawCube(int x, int y, int z,
				Color back, Color front, Color left, Color right, Color bottom, Color top,
				bool highlight=false);		
		/**Set color*/
		void glSetColor(Color col, bool highlight );
};




#endif

